package com.ynfy.buss.course.coursecatalog.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ynfy.buss.course.coursecatalog.entity.CourseCatalog;
import com.ynfy.buss.course.coursecatalog.enums.CatalogType;
import com.ynfy.buss.course.coursecatalog.enums.ResourceType;
import com.ynfy.buss.course.coursecatalog.mapper.CourseCatalogMapper;
import com.ynfy.buss.course.coursecatalog.service.ICourseCatalogService;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.util.CommonUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: course_catalog
 * @Author: jeecg-boot
 * @Date: 2023-10-24
 * @Version: V1.0
 */
@Service
public class CourseCatalogServiceImpl extends ServiceImpl<CourseCatalogMapper, CourseCatalog> implements ICourseCatalogService {

    /**
     * 删除
     *
     * @param courseId
     */
    @Override
    public void remove(String courseId) {
        QueryWrapper<CourseCatalog> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(CourseCatalog::getCourseId, courseId);
        this.remove(wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveAll(String courseId, List<CourseCatalog> catalogList) {
        remove(courseId);
        List<CourseCatalog> newCatalogList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(catalogList)) {
            int sort = 1;
            for (CourseCatalog item : catalogList) {
                //根节点
                item.setCourseId(courseId);
                item.setSort(sort);
                if (ResourceType.VIDEO.getCode().equals(item.getResourceType())) {//如果是视频，则计算视频时长
                    item.setVideoHour(CommonUtils.getVideoDuration(item.getResourceId()));
                } else {
                    item.setVideoHour(null);
                }
                sort++;
                newCatalogList.add(item);

                //子节点
                if (!CollectionUtils.isEmpty(item.getChildList())) {
                    int index = 1;
                    for (CourseCatalog child : item.getChildList()) {
                        if (ResourceType.VIDEO.getCode().equals(child.getResourceType())) {
                            child.setVideoHour(CommonUtils.getVideoDuration(child.getResourceId()));
                        } else {
                            child.setVideoHour(null);
                        }
                        child.setCourseId(courseId);
                        child.setSort(index);
                        newCatalogList.add(child);
                        index++;
                    }
                }
            }
            saveBatch(newCatalogList);
        }
    }

    /**
     * 根据课程id获取目录
     *
     * @param courseId
     * @return
     */
    @Override
    public List<CourseCatalog> listByCourseId(String courseId) {
        QueryWrapper<CourseCatalog> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(CourseCatalog::getCourseId, courseId);
        List<CourseCatalog> courseCatalogList = list(wrapper);
        if (!CollectionUtils.isEmpty(courseCatalogList)) {
            //获取根节点
            List<CourseCatalog> rootList = courseCatalogList.stream().filter(courseCatalog ->
                            StringUtils.isBlank(courseCatalog.getParentId())).sorted(Comparator.comparing(CourseCatalog::getSort))
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(rootList)) {
                rootList.forEach(root -> {
                    //如果根节点是目录，获取任务子节点
                    if (CatalogType.CATALOG.getCode().equals(root.getType())) {
                        List<CourseCatalog> childList = courseCatalogList.stream().filter(courseCatalog ->
                                StringUtils.isNotBlank(courseCatalog.getParentId()) && courseCatalog.getParentId().equals(root.getId())
                        ).sorted(Comparator.comparing(CourseCatalog::getSort)).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(childList)) {
                            if (CollectionUtils.isEmpty(root.getChildList())) {
                                root.setChildList(new ArrayList<>());
                            }
                            root.getChildList().addAll(childList);
                        }
                    }
                });
            }
            return rootList;
        }
        return Collections.emptyList();
    }

}
